/**
 * Copyright Marcus Britanicus <marcusbritanicus@gmail.com>
 *
 * This file is a part of LibDesQ (https://gitlab.com/DesQ/libdesq)
 * This library contains various  Core and Gui classes which are used
 * through out the DesQ Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

#pragma once

/* For QString, QThread, QTimer, etc.. */
#include <QtCore>

#include <QLocalServer>
#include <QLocalSocket>

namespace DesQ {
    namespace IPC {
        class Server;
        class Client;
    }
}

class DesQ::IPC::Server : public QLocalServer {
    Q_OBJECT;

    public:
        Server( QObject *parent = nullptr );

        /*
         * Broadcast a message to all listeners
         * This is similar to emit signal( ... )
         */
        void broadcast( QByteArray );

    private:
        QList<DesQ::IPC::Client *> bcastClients;     // Client to which messages are broadcast
        QList<DesQ::IPC::Client *> xchngClients;     // Clients with which we will exchange data

        QQueue<DesQ::IPC::Client *> pending;

        /* Accept a new connection and handle it suitably */
        void handleIncomingConnection();

    protected:
        void incomingConnection( quintptr socketDescriptor ) override;

    Q_SIGNALS:
        /* Relay the commands */
        void messageReceived( QString );
};
